# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

import logging
import os
import re
from typing import Tuple

from agenteval import jinja_env
from agenteval.evaluators import BaseEvaluator
from agenteval.evaluators.bedrock_request.bedrock_request_handler import (
    BedrockRequestHandler,
)
from agenteval.test import TestResult

logger = logging.getLogger(__name__)

_PROMPT_TEMPLATE_ROOT = "evaluators/canonical"
_SYSTEM_PROMPT_DIR = "system"
_RUNTIME_PROMPT_DIR = "runtime"
_PROMPT_TEMPLATE_NAMES = [
    "generate_initial_prompt",
    "generate_user_response",
    "generate_test_status",
    "generate_evaluation",
]

# enable backwards-compatible StrEnum
try:
    from enum import StrEnum
except ImportError:
    from enum import Enum

    class StrEnum(str, Enum):
        pass


class TestStatusCategories(StrEnum):
    ALL_STEPS_ATTEMPTED = "A"
    NOT_ALL_STEPS_ATTEMPTED = "B"


class EvaluationCategories(StrEnum):
    ALL_EXPECTED_RESULTS_OBSERVED = "A"
    NOT_ALL_EXPECTED_RESULTS_OBSERVED = "B"


class Results(StrEnum):
    MAX_TURNS_REACHED = "Maximum turns reached."
    ALL_EXPECTED_RESULTS_OBSERVED = (
        "All of the expected results can be observed in the conversation."
    )
    NOT_ALL_EXPECTED_RESULTS_OBSERVED = (
        "Not all of the expected results can be observed in the conversation."
    )


class CanonicalEvaluator(BaseEvaluator):
    """An evaluator based on the canoncial templates. Compatible with the model providers supported in BedrockModelConfig"""

    def __init__(
        self,
        **kwargs,
    ):
        """Initialize the evaluator."""
        super().__init__(**kwargs)

        self._prompt_template_map = {
            name: {
                "system": jinja_env.get_template(
                    os.path.join(
                        _PROMPT_TEMPLATE_ROOT, _SYSTEM_PROMPT_DIR, f"{name}.jinja"
                    )
                ),
                "prompt": jinja_env.get_template(
                    os.path.join(
                        _PROMPT_TEMPLATE_ROOT, _RUNTIME_PROMPT_DIR, f"{name}.jinja"
                    )
                ),
            }
            for name in _PROMPT_TEMPLATE_NAMES
        }

    @staticmethod
    def _extract_content_from_xml(xml_data: str, element_names: list[str]) -> Tuple:
        content = []
        for e in element_names:
            pattern = rf"<{e}>(.*?)</{e}>"
            match = re.search(pattern, xml_data, re.DOTALL)
            content.append(match.group(1).strip() if match else None)
        return tuple(content)

    def _generate(
        self,
        system_prompt: str,
        prompt: str,
        output_xml_element: str,
    ) -> str:
        request_body = BedrockRequestHandler.build_request_body(
            request_body=self.model_config.request_body,
            model_config=self.model_config,
            system_prompt=system_prompt,
            prompt=prompt,
        )

        response = self.invoke_model(request_body=request_body)

        completion = BedrockRequestHandler.parse_completion_from_response(
            response=response, model_config=self.model_config
        )

        logger.debug(
            f"[{self.test.name}]\n[PROMPT]\n{prompt}\n[COMPLETION]\n{completion}"
        )

        output, reasoning = self._extract_content_from_xml(
            completion, [output_xml_element, "thinking"]
        )

        return output, reasoning

    def _generate_initial_prompt(self) -> str:
        system_prompt = self._prompt_template_map["generate_initial_prompt"][
            "system"
        ].render()
        prompt = self._prompt_template_map["generate_initial_prompt"]["prompt"].render(
            step=self.test.steps[0]
        )

        initial_prompt, reasoning = self._generate(
            system_prompt=system_prompt,
            prompt=prompt,
            output_xml_element="initial_prompt",
        )

        self.trace.add_step(
            system_prompt=system_prompt,
            prompt=prompt,
            initial_prompt=initial_prompt,
            reasoning=reasoning,
        )
        return initial_prompt

    def _generate_test_status(self) -> str:
        system_prompt = self._prompt_template_map["generate_test_status"][
            "system"
        ].render()
        prompt = self._prompt_template_map["generate_test_status"]["prompt"].render(
            steps=self.test.steps, conversation=self.conversation
        )
        test_status, reasoning = self._generate(
            system_prompt=system_prompt,
            prompt=prompt,
            output_xml_element="category",
        )
        self.trace.add_step(
            system_prompt=system_prompt,
            prompt=prompt,
            test_status=test_status,
            reasoning=reasoning,
        )
        return test_status

    def _generate_evaluation(self) -> tuple[str, str]:
        system_prompt = self._prompt_template_map["generate_evaluation"][
            "system"
        ].render()
        prompt = self._prompt_template_map["generate_evaluation"]["prompt"].render(
            expected_results=self.test.expected_results,
            conversation=self.conversation,
        )

        evaluation, reasoning = self._generate(
            system_prompt=system_prompt,
            prompt=prompt,
            output_xml_element="category",
        )
        self.trace.add_step(
            system_prompt=system_prompt,
            prompt=prompt,
            evaluation=evaluation,
            reasoning=reasoning,
        )

        return evaluation, reasoning

    def _generate_user_response(self) -> str:
        system_prompt = self._prompt_template_map["generate_user_response"][
            "system"
        ].render()
        prompt = self._prompt_template_map["generate_user_response"]["prompt"].render(
            steps=self.test.steps, conversation=self.conversation
        )

        user_response, reasoning = self._generate(
            system_prompt=system_prompt,
            prompt=prompt,
            output_xml_element="user_response",
        )

        self.trace.add_step(
            system_prompt=system_prompt,
            prompt=prompt,
            user_response=user_response,
            reasoning=reasoning,
        )
        return user_response

    def _invoke_target(self, user_input) -> str:
        target_response = self.target.invoke(user_input)
        self.trace.add_step(data=target_response.data)

        return target_response.response

    def evaluate(self) -> TestResult:
        """Conduct the test.

        Returns:
            TestResult
        """
        passed = False
        result = Results.MAX_TURNS_REACHED.value
        reasoning = ""

        while self.conversation.turns < self.test.max_turns:
            if self.conversation.turns == 0:
                # start conversation
                if self.test.initial_prompt:
                    user_input = self.test.initial_prompt
                else:
                    user_input = self._generate_initial_prompt()
            else:
                # generate next user response
                user_input = self._generate_user_response()

            # add turn to the conversation
            self.conversation.add_turn(user_input, self._invoke_target(user_input))

            # get test status
            test_status = self._generate_test_status()
            if test_status == TestStatusCategories.ALL_STEPS_ATTEMPTED:
                # evaluate conversation
                eval_category, reasoning = self._generate_evaluation()
                if (
                    eval_category
                    == EvaluationCategories.NOT_ALL_EXPECTED_RESULTS_OBSERVED.value  # noqa: W503
                ):
                    result = Results.NOT_ALL_EXPECTED_RESULTS_OBSERVED.value
                else:
                    result = Results.ALL_EXPECTED_RESULTS_OBSERVED.value
                    passed = True

                break

        return TestResult(
            test_name=self.test.name,
            passed=passed,
            result=result,
            reasoning=reasoning,
            conversation=self.conversation,
        )



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

import json
from typing import Optional

from jsonpath_ng import parse

from agenteval.targets import Boto3Target, TargetResponse

_SERVICE_NAME = "sagemaker-runtime"
_CONTENT_TYPE = "application/json"
_ACCEPT = "application/json"


class SageMakerEndpointTarget(Boto3Target):
    """A target encapsulating an Amazon SageMaker endpoint."""

    def __init__(
        self,
        endpoint_name: str,
        request_body: dict,
        input_path: str,
        output_path: str,
        custom_attributes: Optional[str] = None,
        target_model: Optional[str] = None,
        target_variant: Optional[str] = None,
        target_container_hostname: Optional[str] = None,
        inference_component_name: Optional[str] = None,
        **kwargs
    ):
        """
        Initialize the target.

        Args:
            endpoint_name (str): The name of the Amazon SageMaker endpoint.
            request_body (dict): The data that is sent to the endpoint, which includes
                a placeholder for the prompt. During a run, the placeholder will be
                replaced by a prompt generated by the Evaluator.
            input_path (str): A JSONPath expression to match the field for the input
                prompt in the request body.
            output_path (str): A JSONPath expression to match the generated text in
                the response body.
            custom_attributes (Optional[str]): Provides additional information about a request
                for an inference submitted to a model hosted at an Amazon SageMaker endpoint.
            target_model (Optional[str]): The model to request for inference when invoking a
                multi-model endpoint.
            target_variant (Optional[str]): The production variant to send the inference request
                to when invoking an endpoint that is running two or more variants.
            target_container_hostname (Optional[str]): The hostname of the container to invoke if
                the endpoint hosts multiple containers and is configured to use direct invocation.
            inference_component_name (Optional[str]): The name of the inference component to invoke
                if the endpoint hosts one or more inference components.
        """
        super().__init__(boto3_service_name=_SERVICE_NAME, **kwargs)

        self._request_body = request_body
        self._input_jp_expr = parse(input_path)
        self._output_jp_expr = parse(output_path)

        self._args = self._create_base_args(
            endpoint_name,
            custom_attributes,
            target_model,
            target_variant,
            target_container_hostname,
            inference_component_name,
        )

    @staticmethod
    def _create_base_args(
        endpoint_name: str,
        custom_attributes: Optional[str],
        target_model: Optional[str],
        target_variant: Optional[str],
        target_container_hostname: Optional[str],
        inference_component_name: Optional[str],
    ) -> dict:
        args = {
            "EndpointName": endpoint_name,
            "ContentType": _CONTENT_TYPE,
            "Accept": _ACCEPT,
            **{
                key: value
                for key, value in {
                    "CustomAttributes": custom_attributes,
                    "TargetModel": target_model,
                    "TargetVariant": target_variant,
                    "TargetContainerHostname": target_container_hostname,
                    "InferenceComponentName": inference_component_name,
                }.items()
                if value is not None
            },
        }

        return args

    def _update_request(self, prompt: str):
        self._input_jp_expr.update(self._request_body, prompt)
        self._args["Body"] = json.dumps(self._request_body)

    def _query_response(self, response_body: dict) -> str:
        return self._output_jp_expr.find(response_body)[0].value

    def invoke(self, prompt: str) -> TargetResponse:
        """Invoke the target with a prompt.

        Args:
            prompt (str): The prompt as a string.

        Returns:
            TargetResponse
        """
        self._update_request(prompt)

        response = self.boto3_client.invoke_endpoint(**self._args)

        response_body = json.loads(response.get("Body").read())

        return TargetResponse(response=self._query_response(response_body))



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, computed_field, model_validator

from agenteval import defaults
from agenteval.test import Test


class TestSuite(BaseModel):
    """A collection of tests to be executed.

    Attributes:
        tests: A list of tests.
    """

    tests: list[Test]

    # do not collect as a pytest
    __test__ = False

    @model_validator(mode="after")
    def _check_test_names_unique(self) -> TestSuite:
        unique_names = len(set(test.name for test in self.tests))

        if unique_names != len(self.tests):
            raise ValueError("Test names must be unique")

        return self

    def __iter__(self):
        return iter(self.tests)

    @computed_field
    @property
    def num_tests(self) -> int:
        """Returns the number of tests in the test suite."""
        return len(self.tests)

    @classmethod
    def load(cls, config: dict[str, dict], filter: Optional[str]) -> TestSuite:
        """Loads a `TestSuite` from a list of test configurations and an optional filter.

        Args:
            config (dict[str, dict]): A dictionary of test configurations, where
                the keys are the test names and the values are the test cases as dictionaries.
            filter (Optional[str]): A filter string to apply when loading the tests.

        Returns:
            TestSuite: A `TestSuite` instance containing the loaded tests.
        """
        return cls(tests=TestSuite._load_tests(config, filter))

    @staticmethod
    def _load_tests(config: dict[str, dict], filter: Optional[str]) -> list[Test]:
        tests = []

        if filter:
            names = TestSuite._parse_filter(filter)
        else:
            names = config.keys()

        for name in names:
            tests.append(
                Test(
                    name=name,
                    steps=config[name]["steps"],
                    expected_results=config[name]["expected_results"],
                    initial_prompt=config[name].get("initial_prompt"),
                    max_turns=config[name].get("max_turns", defaults.MAX_TURNS),
                    hook=config[name].get("hook"),
                )
            )

        return tests

    @staticmethod
    def _parse_filter(filter: str) -> list[str]:
        return [n.strip() for n in filter.split(",")]



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

from pydantic import BaseModel

from agenteval.evaluators import BaseEvaluator
from agenteval.evaluators.canonical.evaluator import CanonicalEvaluator
from agenteval.evaluators.model_config.bedrock_model_config import BedrockModelConfig
from agenteval.evaluators.model_config.preconfigured_model_configs import (
    DEFAULT_CLAUDE_3_5_MODEL_CONFIG,
    DEFAULT_CLAUDE_3_MODEL_CONFIG,
    DEFAULT_CLAUDE_HAIKU_3_5_US_MODEL_CONFIG,
    DEFAULT_CLAUDE_US_3_7_MODEL_CONFIG,
    DEFAULT_LLAMA_3_3_70B_US_MODEL_CONFIG,
)
from agenteval.targets import BaseTarget
from agenteval.test import Test

_EVALUATOR_METHOD_MAP = {
    "canonical": CanonicalEvaluator,
}

_DEFAULT_MODEL_CONFIG_MAP = {
    "claude-3": DEFAULT_CLAUDE_3_MODEL_CONFIG,
    "claude-3_5": DEFAULT_CLAUDE_3_5_MODEL_CONFIG,
    "claude-3_7-us": DEFAULT_CLAUDE_US_3_7_MODEL_CONFIG,
    "claude-haiku-3_5-us": DEFAULT_CLAUDE_HAIKU_3_5_US_MODEL_CONFIG,
    "llama-3_3-us": DEFAULT_LLAMA_3_3_70B_US_MODEL_CONFIG,
}


class EvaluatorFactory(BaseModel):
    """A factory for creating instances of `BaseEvaluator` subclasses.

    Attributes:
        config: A dictionary containing the configuration parameters
            needed to create a `BaseEvaluator` instance.
    """

    config: dict

    def create(self, test: Test, target: BaseTarget, work_dir: str) -> BaseEvaluator:
        """Create an instance of the evaluator class specified in the configuration.

        Args:
            test (Test): The test case.
            target (BaseTarget): The target agent being evaluated.
            work_dir (str): The directory where the test result and trace will be
                generated.

        Returns:
            BaseEvaluator: An instance of the evaluator class, with the configuration
                parameters applied.
        """
        reserved_config_keys = {"eval_method", "model", "custom_config"}
        evaluator_cls = self._get_evaluator_class()
        return evaluator_cls(
            test=test,
            target=target,
            work_dir=work_dir,
            model_config=self._get_bedrock_model_config(),
            **{k: v for k, v in self.config.items() if k not in reserved_config_keys},
        )

    def _get_evaluator_class(self) -> type[BaseEvaluator]:
        if "eval_method" in self.config:
            return _EVALUATOR_METHOD_MAP[self.config["eval_method"]]
        else:
            return CanonicalEvaluator

    """
    If the use passes in both an id and a request body, create a custom config instance; otherwise use a default config for the specified model
    """

    def _get_bedrock_model_config(self) -> BedrockModelConfig:
        if "custom_config" in self.config:
            return BedrockModelConfig(
                model_id=self.config["custom_config"]["model_id"],
                request_body=self.config["custom_config"]["request_body"],
            )
        else:
            return _DEFAULT_MODEL_CONFIG_MAP[self.config["model"]]



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

import inspect
import json
import os
from datetime import datetime, timezone
from typing import Optional

_TRACE_DIR = "agenteval_traces"


class Trace:
    """A context manager which captures steps taken during evaluation.

    Once the context manager exits, the trace is dumped to a JSON file.

    Attributes:
        test_name (str): Name of the test.
        trace_dir (str): Directory to store the trace.
        start_time (datetime): Start time of the trace.
        end_time (datetime): End time of the trace.
        steps (list): List of steps in the trace.

    """

    def __init__(self, test_name: str, work_dir: str):
        """
        Initialize the trace handler.

        Args:
            test_name (str): Name of the test.
            work_dir (str): Directory to store the trace.
        """
        self.test_name = test_name
        self.trace_dir = os.path.join(work_dir, _TRACE_DIR)
        self.start_time = None
        self.end_time = None
        self.steps = []

    def __enter__(self):
        self.start_time = datetime.now(timezone.utc)
        return self

    def __exit__(self, *exc):
        self.end_time = datetime.now(timezone.utc)
        self._dump_trace()

    def _dump_trace(self):
        os.makedirs(self.trace_dir, exist_ok=True)

        with open(os.path.join(self.trace_dir, f"{self.test_name}.json"), "w") as f:
            json.dump(self._get_trace(), f, default=str)

    def _get_trace(self) -> str:
        return {
            "test_name": self.test_name,
            "start_time": self.start_time,
            "end_time": self.end_time,
            "steps": self.steps,
        }

    def add_step(self, step_name: Optional[str] = None, **kwargs):
        """Add a step to the trace.

        Args:
            step_name (Optional[str]): The name of the step. Defaults to
                the name of the caller function
        """
        step_name = step_name or inspect.stack()[1].function
        step = {"timestamp": datetime.now(timezone.utc), "step_name": step_name}
        step.update(kwargs)
        self.steps.append(step)



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from abc import ABC, abstractmethod

from agenteval.targets import TargetResponse


class BaseTarget(ABC):
    """Defines the common interface for target classes."""

    @abstractmethod
    def invoke(self, prompt: str) -> TargetResponse:
        """Invoke the target with a prompt.

        Args:
            prompt (str): The prompt as a string.

        Returns:
            TargetResponse
        """
        pass



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

from .bedrock_request_handler import BedrockRequestHandler

__all__ = ["BedrockRequestHandler"]



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

from pydantic import BaseModel

from agenteval.targets import BaseTarget
from agenteval.targets.bedrock_agent import BedrockAgentTarget
from agenteval.targets.bedrock_flow import BedrockFlowTarget
from agenteval.targets.bedrock_knowledge_base import BedrockKnowledgeBaseTarget
from agenteval.targets.lexv2 import LexV2Target
from agenteval.targets.q_business import QBusinessTarget
from agenteval.targets.sagemaker_endpoint import SageMakerEndpointTarget
from agenteval.targets.weni import WeniTarget
from agenteval.utils import import_class

_TARGET_MAP = {
    "bedrock-agent": BedrockAgentTarget,
    "bedrock-flow": BedrockFlowTarget,
    "q-business": QBusinessTarget,
    "sagemaker-endpoint": SageMakerEndpointTarget,
    "bedrock-knowledge-base": BedrockKnowledgeBaseTarget,
    "lex-v2": LexV2Target,
    "weni": WeniTarget,
}


class TargetFactory(BaseModel):
    """A factory for creating instances of `BaseTarget` subclasses.

    Attributes:
        config: A dictionary containing the configuration parameters
            needed to create a `BaseTarget` instance.
    """

    config: dict

    def create(self) -> BaseTarget:
        """Create an instance of the target class specified in the configuration.

        Returns:
            BaseTarget: An instance of the target class, with the configuration
                parameters applied.
        """
        target_cls = self._get_target_class()

        return target_cls(**{k: v for k, v in self.config.items() if k != "type"})

    def _get_target_class(self) -> type[BaseTarget]:
        if self.config["type"] in _TARGET_MAP:
            target_cls = _TARGET_MAP[self.config["type"]]
        else:
            target_cls = import_class(self.config["type"], parent_class=BaseTarget)

        return target_cls



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0


def calculate_pass_rate_metric(pass_count: int, num_tests: int) -> float:
    """Calculate the pass rate metric.

    Args:
        pass_count (int): The number of tests that passed.
        num_tests (int): The total number of tests.

    Returns:
        float: The pass rate metric.
    """
    return round((pass_count / num_tests) * 100, 2)



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

import os
import json
import logging
from pathlib import Path
from typing import Optional

logger = logging.getLogger(__name__)

STORE_TOKEN_KEY = "token"
STORE_PROJECT_UUID_KEY = "project_uuid"
STORE_WENI_BASE_URL = "weni_base_url"
STORE_NEXUS_BASE_URL = "nexus_base_url"
STORE_KEYCLOAK_URL = "keycloak_url"
STORE_KEYCLOAK_REALM = "keycloak_realm"
STORE_KEYCLOAK_CLIENT_ID = "keycloak_client_id"
STORE_CLI_BASE_URL = "cli_base_url"


class Store:
    """
    A utility class to interact with the Weni CLI cache file.
    
    This class provides methods to read configuration values (like tokens and project UUIDs)
    from the Weni CLI cache file, which is typically stored at ~/.weni_cli
    """
    
    def __init__(self):
        self.file_path = f"{Path.home()}{os.sep}.weni_cli"
        logger.debug(f"Store initialized with file path: {self.file_path}")

    def get(self, key: str, default: Optional[str] = None) -> Optional[str]:
        """
        Get a value from the store by key.
        
        Args:
            key (str): The key to look up
            default (Optional[str]): Default value if key is not found
            
        Returns:
            Optional[str]: The value associated with the key, or default if not found
        """
        try:
            if not os.path.exists(self.file_path):
                logger.debug(f"Store file does not exist at {self.file_path}")
                return default
                
            with open(self.file_path, "r") as file:
                content = file.read().strip()
                if not content:
                    logger.debug("Store file is empty")
                    return default
                    
                data = json.loads(content)
                value = data.get(key, default)
                logger.debug(f"Retrieved value for key '{key}': {'***' if 'token' in key.lower() else value}")
                return value
                
        except (json.JSONDecodeError, IOError) as e:
            logger.warning(f"Error reading from store file: {e}")
            return default

    def get_token(self) -> Optional[str]:
        """
        Get the Weni authentication token from the store.
        
        Returns:
            Optional[str]: The authentication token, or None if not found
        """
        return self.get(STORE_TOKEN_KEY)

    def get_project_uuid(self) -> Optional[str]:
        """
        Get the Weni project UUID from the store.
        
        Returns:
            Optional[str]: The project UUID, or None if not found
        """
        return self.get(STORE_PROJECT_UUID_KEY)



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

from agenteval.targets import Boto3Target, TargetResponse

_SERVICE_NAME = "bedrock-agent-runtime"


class BedrockFlowTarget(Boto3Target):
    """A target encapsulating an Amazon Bedrock Flow."""

    def __init__(self, bedrock_flow_id: str, bedrock_flow_alias_id: str, **kwargs):
        """Initialize the target.

        Args:
            bedrock_flow_id (str): The unique identifier of the Bedrock flow.
            bedrock_flow_alias_id (str): The alias of the Bedrock flow.
        """
        super().__init__(boto3_service_name=_SERVICE_NAME, **kwargs)
        self._bedrock_flow_id = bedrock_flow_id
        self._bedrock_flow_alias_id = bedrock_flow_alias_id

    def invoke(self, prompt: str) -> TargetResponse:
        """Invoke the target with a prompt.

        Args:
            prompt (str): The prompt as a string.

        Returns:
            TargetResponse
        """
        args = {
            "enableTrace": True,
            "flowIdentifier": self._bedrock_flow_id,
            "flowAliasIdentifier": self._bedrock_flow_alias_id,
            "inputs": [
                {
                    # Although the API implies otherwise, Flows currently seem to be limited to
                    # follow this single-input node pattern anyway:
                    "content": {"document": prompt},
                    "nodeName": "FlowInputNode",
                    "nodeOutputName": "document",
                }
            ],
        }

        response = self.boto3_client.invoke_flow(**args)

        stream = response["responseStream"]
        completion = ""
        trace_data = []

        for event in stream:
            if "flowTraceEvent" in event:
                trace_data.append(event["flowTraceEvent"].get("trace"))
            if "flowOutputEvent" in event:
                output_event = event["flowOutputEvent"]
                # Testing 2024-12 suggests 'nodeType' actually not always present in API (despite
                # the docs?):
                if (
                    "nodeType" not in output_event
                    or output_event["nodeType"] == "FlowOutputNode"
                ):
                    completion += output_event.get("content", {}).get("document", "")

            errs = {k: v for k, v in event.items() if k.endswith("Exception")}
            if errs:
                raise ValueError(errs)

        return TargetResponse(
            response=completion, data={"bedrock_flow_trace": trace_data}
        )



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0
from dataclasses import dataclass
from enum import Enum
from typing import Dict


class ModelProvider(Enum):
    META = "meta"
    ANTHROPIC = "anthropic"


@dataclass
class BedrockModelConfig:
    model_id: str
    request_body: Dict

    """
    Match the provider by looking for the provider name in the model_id.
    This should be a robust approach, but if name conflicts are found in the future,
    constructors can explicitly provide the ModelProvider directly.
    Even for provisioned throughput ARN, the modelID will adhere to the matching defined here.
    """

    @property
    def provider(self) -> ModelProvider:
        if "meta" in self.model_id:
            return ModelProvider.META
        elif "anthropic" in self.model_id:
            return ModelProvider.ANTHROPIC
        else:
            raise ValueError(f"Unsupported model ID: {self.model_id}")



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

from agenteval.test import Test, TestResult
from agenteval.trace import Trace


class Hook:
    """An evaluation hook."""

    def pre_evaluate(test: Test, trace: Trace) -> None:
        """
        Method called before evaluation. Can be used to perform any setup tasks.

        Args:
            test (Test): The test case.
            trace (Trace): Captures steps during evaluation.
        """
        pass

    def post_evaluate(test: Test, test_result: TestResult, trace: Trace) -> None:
        """
        Method called after evaluation. This may be used to perform integration testing
        or clean up tasks.

        Args:
            test (Test): The test case.
            test_result (TestResult): The result of the test, which can be overriden
                by updating the attributes of this object.
            trace (Trace): Captures steps during evaluation.
        """
        pass



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

from typing import Optional

from pydantic import BaseModel


class TargetResponse(BaseModel):
    """A target's response.

    Attributes:
        response: The response string.
        data: Additional data (if applicable).
    """

    response: str
    data: Optional[dict] = None



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

from importlib import import_module
from typing import Optional

_ALLOWED_MODULE_NAME_SUFFIX = ["_hook", "_target"]


def import_class(module_path: str, parent_class: Optional[type] = None) -> type:
    """Import a class from a module path and optionally validate
    that it is a subclass of a given parent class.

    Args:
        module_path (str): The full module path to the class, e.g. "my.module.ClassName".
        parent_class (Optional[type]): A parent class to validate the imported class against.

    Returns:
        type

    Raises:
        ValueError: If the module name does not end with one of the allowed suffixes.
        TypeError: If imported class is not a subclass of the provided parent class.
    """

    name, class_name = module_path.rsplit(".", 1)

    # make sure module name starts with one of the allowed suffixes
    _validate_module_name(name.split(".")[-1])

    module = import_module(name)
    cls = getattr(module, class_name)

    if parent_class:
        # make sure the imported class is a subclass
        _validate_subclass(cls, parent_class)

    return cls


def _validate_module_name(name: str):
    if not any(name.endswith(suffix) for suffix in _ALLOWED_MODULE_NAME_SUFFIX):
        raise ValueError(f"Invalid module name: {name}")


def _validate_subclass(child_class: type, parent_class: type):
    if not issubclass(child_class, parent_class):
        raise TypeError(
            f"{child_class.__name__} is not a {parent_class.__name__} subclass"
        )
